from setuptools import setup, find_packages

with open('README.md', 'r') as readme:
    long_desc = readme.read()

setup(
    name='pyinstantcrawl',
    version='1.0.1',
    description='scrape a website with json template',
    long_description=long_desc,
    url='https://github.com/ihfazhillah/py-instant-crawl',
    author='ihfazhillah',
    author_email='me@ihfazh.com',
    license='MIT',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'Click==7.0',
        'parsel==1.5.1',
        'requests==2.21.0'
    ],
    entry_points={
        'console_scripts': [
            'pyinstantcrawl=pyinstantcrawl.main:fetch'
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3 :: Only'
    ]
)



import json
import click
import pyinstantcrawl


@click.command()
@click.argument('url', nargs=1)
@click.argument('template', nargs=1, type=click.File('r'))
def fetch(url, template):
    crawled = pyinstantcrawl.fetch(url, template.read())
    print(json.dumps(crawled, indent=4))


if __name__ == "__main__":
    fetch()



import json

import requests
from parsel import Selector
import pprint


sample_json = """
{
    "tip-of-day": {
        "expression": "string(//div[@class='tip-of-day'])",
        "type": "xpath",
        "getter": "get"
    },
    "testimonial": {
        "expression": ".testimonial",
        "type": "css",
        "getter": "getall"
    }
}
"""

sample_with_child = """
{
    "contents": {
        "expression": "//section[@id='content']/ul/li",
        "type": "xpath",
        "children": {
            "datetime": {
                "expression": "./time/text()",
                "type": "xpath",
                "getter": "get"
            },
            "url": {
                "expression": "./a/@href",
                "type": "xpath",
                "getter": "get"
            },
            "title": {
                "expression": "./a/text()",
                "type": "xpath",
                "getter": "get"
            }
        }
    }
}
"""


def parse_node(selector, selector_data):
    """ Parse the node

    :selector: parsel.Selector instance
    :selector_data: a dictionary object contains expression, type, and getter
    :returns: string

    """
    stype = selector_data['type']
    expression = selector_data['expression']
    getter = selector_data.get('getter', 'get')
    selectlist = getattr(selector, stype)(expression)

    return getattr(selectlist, getter)(), selectlist


def parse(result, selector, json_ob, is_child=False, parent=None):
    """doing real parsing

    :selector: parsel.Selector instance
    :json_string: the whole json scraper template
    :returns: dict

    """
    for key, val in json_ob.items():
        children = val.get('children')

        if not children:
            res, _ = parse_node(selector, val)
            result[key] = res
        else:
            _, p_selector = parse_node(selector, val)
            l_result = []
            for s in p_selector:
                with_child = parse({}, s, children)
                l_result.append(with_child)
            result[key] = l_result

    return result


def fetch(url, template):
    """Do actual fetch and return the selected data

    :url: TODO
    :template: TODO
    :returns: TODO

    """
    result = {}
    resp = requests.get(url)
    selector = Selector(resp.text)
    json_ob = json.loads(template)
    parsed = parse(result, selector, json_ob)
    return parsed


if __name__ == "__main__":
    print(fetch("https://pragprog.com", sample_json))
    print("nested")
    pprint.pprint(
        fetch("https://blog.ihfazh.com/archives.html", sample_with_child))
